package org.jeecg.modules.app.controller;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.modules.app.service.IOrderService;
import org.jeecg.modules.app.vo.OrderVO;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

@Slf4j
@Tag(name = "订单相关接口")
@RestController
@RequestMapping("/app/order")
public class OrderController {
    @Resource
    private IOrderService appOrderService;
    /**
     * @Author SheepHy
     * @Description 学校场地预约-预览页
     * @Date 10:10 2025/7/14
     * @Param placeId startTime
     * @return Result<OrderVO.PreviewOrderPlaceSchool>
     **/
    @Operation(summary="学校场地预约-预览页")
    @GetMapping("/previewOrderPlaceSchool")
    public Result<OrderVO.PreviewOrderPlaceSchool> previewOrderPlaceSchool(@RequestParam("placeId") @Schema(description = "场地ID")String placeId,
                                                                           @RequestParam("startTime") @Schema(description = "当前选择日期（年月日）")Date startTime){
        return Result.ok(appOrderService.previewOrderPlaceSchool(placeId,startTime));
    }

    /**
     * @Author SheepHy
     * @Description 课程限购校验
     * @Date 16:22 2025/7/14
     * @Param courseId 课程ID
     * @return boolean
     **/
    @Operation(summary="课程限购校验")
    @GetMapping("/checkCourseLimitNum")
    public Result<Boolean> checkCourseLimitNum(@RequestParam("courseId") @Schema(description = "课程ID")String courseId,
                                               @RequestParam("limitNum") @Schema(description = "购买数")int limitNum){
        return Result.OK(appOrderService.checkCourseLimitNum(courseId, limitNum));
    }

    /**
     * @Author SheepHy
     * @Description 课程购买-预览页（学校课程、体育馆课程、教练课程）
     * @Date 15:43 2025/7/14
     * @Param courseId
     * @return OrderVO.PreviewOrderCourseSchool {@link OrderVO.PreviewOrderCourse}
     **/
    @Operation(summary="课程购买-预览页（学校课程、体育馆课程、教练课程）")
    @GetMapping("/previewOrderCourse")
    public Result<OrderVO.PreviewOrderCourse> previewOrderCourse(@RequestParam("courseId") @Schema(description = "课程ID")String courseId){
        return Result.ok(appOrderService.previewOrderCourse(courseId));
    }

    /**
     * @Author SheepHy
     * @Description 体育馆包场-预览页
     * @Date 16:13 2025/7/14
     * @Param placeId
     * @return OrderVO.PreviewOrderPlaceGymnasiumChartered {@link OrderVO.PreviewOrderPlaceGymnasiumChartered}
     **/
    @Operation(summary="体育馆包场-预览页")
    @PostMapping("/previewOrderPlaceGymnasiumChartered")
    public Result<OrderVO.PreviewOrderPlaceGymnasiumChartered> previewOrderPlaceGymnasiumChartered(@RequestParam("courseId") @Schema(description = "课程ID")List<String> rulesId){
        return Result.ok(appOrderService.previewOrderPlaceGymnasiumChartered(rulesId));
    }
}
