package org.jeecg.modules.app.service.impl;

import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.extern.log4j.Log4j2;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.system.vo.LoginUser;
import org.jeecg.modules.app.service.IOrderService;
import org.jeecg.modules.app.vo.OrderVO;
import org.jeecg.modules.system.app.entity.AppCourses;
import org.jeecg.modules.system.app.entity.AppOrder;
import org.jeecg.modules.system.app.entity.AppSitePlace;
import org.jeecg.modules.system.app.entity.AppSitePriceRules;
import org.jeecg.modules.system.app.mapper.*;
import org.jeecg.modules.system.entity.SysDepart;
import org.jeecg.modules.system.mapper.SysDepartMapper;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.ZoneId;
import java.util.Date;
import java.util.List;
import java.util.concurrent.ThreadLocalRandom;

@Service
@Log4j2
public class OrderServiceImpl implements IOrderService {
    @Resource
    private AppSitePlaceMapper appSitePlaceMapper;
    @Resource
    private AppSitePriceRulesMapper appSitePriceRulesMapper;
    @Resource
    private AppCoursesMapper appCoursesMapper;
    @Resource
    private AppOrderMapper appOrderMapper;
    @Resource
    private SysDepartMapper sysDepartMapper;
    @Resource
    private AppOrderProductMapper appOrderProductMapper;
    @Resource
    private AppOrderUserMapper appOrderUserMapper;

    @Override
    public OrderVO.createOrderPlaceSchool createOrderPlaceSchool(String rulesId) {

        return null;
    }

    @Override
    public boolean checkCourseLimitNum(String courseId, int limitNum) {
        return appCoursesMapper.selectById(courseId).getLimitNum() <= limitNum;
    }

    @Override
    public OrderVO.PreviewOrderPlaceSchool previewOrderPlaceSchool(String placeId, Date startTime) {
        LocalDate localDate = startTime.toInstant().atZone(ZoneId.systemDefault()).toLocalDate();
        Date dateOnly = Date.from(localDate.atStartOfDay(ZoneId.systemDefault()).toInstant());
        OrderVO.PreviewOrderPlaceSchool previewOrderPlaceSchool = new OrderVO.PreviewOrderPlaceSchool();
        List<OrderVO.PreviewOrderPlaceSchoolChild> timeSlot = appSitePriceRulesMapper.timeSlot(placeId, dateOnly);
        AppSitePlace appSitePlace = appSitePlaceMapper.selectById(placeId);
        previewOrderPlaceSchool.setName(appSitePlace.getName())
                .setId(appSitePlace.getId())
                .setTimeSlot(timeSlot);
        return previewOrderPlaceSchool;
    }

    @Override
    public OrderVO.PreviewOrderCourse previewOrderCourse(String courseId) {
        LoginUser user = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        OrderVO.PreviewOrderCourse previousOrderCourse = new OrderVO.PreviewOrderCourse();
        AppCourses appCourses = appCoursesMapper.selectById(courseId);
        boolean checkCourseDiscount = checkCourseDiscount(appCourses.getCategoryId(), user.getId());
        previousOrderCourse.setId(appCourses.getId())
                .setCover(appCourses.getCover())
                .setName(appCourses.getName())
                .setSellingPrice(appCourses.getSellingPrice())
                .setReminder(appCourses.getReminder())
                .setHasDiscount(checkCourseDiscount);
        if(checkCourseDiscount){
            previousOrderCourse.setDiscountPrice(appCourses.getSellingPrice());
            previousOrderCourse.setTotalDiscount(appCourses.getSellingPrice());
        }else {
            previousOrderCourse.setDiscountPrice(BigDecimal.ZERO);
            previousOrderCourse.setTotalDiscount(BigDecimal.ZERO);
        }
        return previousOrderCourse;
    }

    @Override
    public OrderVO.PreviewOrderPlaceGymnasiumChartered previewOrderPlaceGymnasiumChartered(List<String> rulesId) {
        LoginUser user = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        OrderVO.PreviewOrderPlaceGymnasiumChartered result = new OrderVO.PreviewOrderPlaceGymnasiumChartered();
        List<AppSitePriceRules> priceRules = appSitePriceRulesMapper.selectList(Wrappers.<AppSitePriceRules>lambdaQuery()
                .in(AppSitePriceRules::getId, rulesId));
        BigDecimal totalPrice = priceRules.stream()
                .map(AppSitePriceRules::getSellingPrice)
                .reduce(BigDecimal.ZERO, BigDecimal::add);
        if (priceRules.isEmpty()) {
            throw new IllegalArgumentException("未找到对应的价格规则");
        }
        AppSitePlace sitePlace = appSitePlaceMapper.selectById(priceRules.get(0).getSitePlaceId());
        if (sitePlace == null) {
            throw new IllegalStateException("关联场地不存在");
        }
        List<OrderVO.PreviewOrderTimePeriod> timePeriod = appSitePriceRulesMapper.previewOrderPlaceGymnasiumChartered(rulesId);
        SysDepart depart = sysDepartMapper.selectOne(Wrappers.<SysDepart>lambdaQuery()
                .eq(SysDepart::getOrgCode, sitePlace.getOrgCode()));
        return result.setTimePeriod(timePeriod)
                .setEarlyRefundTime(sitePlace.getEarlyRefundTime())
                .setPlaceId(sitePlace.getId())
                .setPlaceName(sitePlace.getName())
                .setReminder(sitePlace.getReminder())
                .setRefundRule(sitePlace.getEarlyRefundTime())
                .setMobile(user.getPhone())
                .setTotalPrice(totalPrice)  // 总价即为所有价格之和
                .setSubtotal(totalPrice)    // 小计与总价相同
                .setStoreName(depart != null ? depart.getDepartName() : "未知门店");
    }

    /**
     * @Author SheepHy
     * @Description 查询用户对该类型课程是否有试听资格
     * @Date 16:45 2025/7/14
     * @Param courseId userId
     * @return boolean
     **/
    private boolean checkCourseDiscount(String categoryId, String userId) {
        boolean success = Boolean.TRUE;
        List<AppOrder> appOrders = appOrderMapper.selectList(Wrappers.<AppOrder>lambdaQuery()
                .eq(AppOrder::getUserId, userId)
                .ne(AppOrder::getOrderStatus, 6)
                .ne(AppOrder::getOrderStatus, 5)
                .eq(AppOrder::getStatus, 0)
                .eq(AppOrder::getDelFlag, 0)
                .eq(AppOrder::getType,2));
        if (appOrders.isEmpty()) return true;
        for (AppOrder appOrder : appOrders) {
            if (!appOrder.getProductIds().contains(categoryId)) {
                success = false;
                break;
            }
        }
        return success;
    }

    /**
     * @Author SheepHy
     * @Description 根据不同的类型生成订单及订单明细
     * @Date 15:09 2025/7/15
     * @Param id type userId
     * @return
     **/
    private String createOrder(List<String> id, String type, String userId) {
        //订单类型；0场地/1赛事/2课程
        AppOrder appOrder = new AppOrder();
        switch(type){
            case "0":
                List<AppSitePriceRules> priceRules = appSitePriceRulesMapper.selectList(Wrappers.<AppSitePriceRules>lambdaQuery()
                        .in(AppSitePriceRules::getId, id));
                BigDecimal totalPrice = priceRules.stream()
                        .map(AppSitePriceRules::getSellingPrice)
                        .reduce(BigDecimal.ZERO, BigDecimal::add);
                if(priceRules.get(0).getType() == 0){
                    if(totalPrice.compareTo(BigDecimal.ZERO) == 0){
                        appOrder.setPayType(3)
                                .setOrderStatus(2);
                    }else {
                        appOrder.setPayType(2)
                                .setOrderStatus(1);
                    }
                    appOrder.setOrderCode(generateOrderNumber(0))
                            .setOrderCode(priceRules.get(0).getOrgCode())
                            .setTenantId(priceRules.get(0).getTenantId())
                            .setProductIds(priceRules.get(0).getSitePlaceId())
                            .setType(0)
                            .setUserId(userId)
                            .setOriginalPrice(totalPrice);
//                            .setPrice(totalPrice)
                }
                break;
            case "1":
                break;
            case "2":
                break;
        }
        appOrderMapper.insert(appOrder);
        return "";
    }

    /**
     * @Author SheepHy
     * @Description 订单编号生成逻辑
     * @Date 17:18 2025/7/15
     * @return
     **/
    private String generateOrderNumber(int type) {
        String format = DateUtil.format(new Date(), "yyyyMMddHHmmss");
        int nextInt = ThreadLocalRandom.current().nextInt(1000, 10000);
        if(type == 0){
            return "D" + format + nextInt;
        }else {
            return "T" + format + nextInt;
        }
    }
}
